/**
 *
 * Seten faktury pomoc rozhran SAX
 * ukzkov program k lnku XML API vydanm v Softwarovch novinch 3/2002
 * Copyright (c) Jirka Kosek 2002
 *
 * POZOR: Pro sputn je poteba JAXP1.1 (v1.0 nepodporuje SAX2)
 *
 * */

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;

public class SAXUkazka
{
  public static void main(String[] args)
  {
   try
    {
      // vytvoen parseru
      SAXParserFactory spf = SAXParserFactory.newInstance();
      SAXParser parser = spf.newSAXParser();

      // vytvoen handleru
      MyHandler handler = new MyHandler();

      // sputn parseru na soubor
      parser.parse("faktura.xml", handler);

      // vpis statistiky
      System.out.println("Celkem K:  " + handler.getCelkem());
      System.out.println("Celkem DPH: " + handler.getDPH());
    }
    // zachycen vjimek
    catch (SAXException e)
    {
      System.out.println(e.getMessage());
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  static class MyHandler extends DefaultHandler
  {
    // tae pro element a znaky
    double suma;
    double sumaDPH;

    boolean inPolozka;
    boolean inCena;

    StringBuffer castka;
    double sazbaDPH;

    // konstruktor
    public MyHandler()
    {
      // inicializace stavovch promnnch
      suma = 0;
      sumaDPH = 0;
      inPolozka = false;
      inCena = false;
    }

    // funkce vrac poet znak v dokumentu
    public double getCelkem()
    {
      return suma;
    }

    // funkce vrac poet element zadanho jmna
    public double getDPH()
    {
      return sumaDPH;
    }

    // obsluha zatku elementu
    public void startElement(String namespaceURI, String localName,
                             String qName, Attributes atts)
    {
      // zjistme, zda jsme v nov poloce faktury
      if (qName.equals("polozka"))
      {
        inPolozka = true;
        return;
      }

      // zjistme, zda jsme v cen
      if (qName.equals("cena"))
      {
        if (!inPolozka)
          System.out.println("Chyba: cena nen uvedena uvnit poloky!");

        inCena = true;

        // vynulujeme stda znak, kter tvo stku
        castka = new StringBuffer(0);

        // projdeme vechny atributy
        int len = atts.getLength();
        for (int i = 0; i < len; i++)
        {
          // hledme atribut "dph"
          if (atts.getQName(i).equals("dph"))
          {
            // zjistme sazbu DPH
            sazbaDPH = Double.parseDouble(atts.getValue("dph")) / 100;
          }
        }
      }
    }

    // zpracovn konce elementu
    public void endElement(String namespaceURI, String localName,
                           String qName)
    {
      // zjistme, zda jsme na konci poloky faktury
      if (qName.equals("polozka"))
      {
          inPolozka = false;
          return;
      }

      // zjistme, zda jsme na konci ceny a v poloce
      if (qName.equals("cena") && inPolozka)
      {
          inCena = false;

          // provedeme vpoty
          suma += Double.parseDouble(castka.toString());
          sumaDPH += sazbaDPH * Double.parseDouble(castka.toString());

          return;
      }
    }

    // obsluha znakovch dat
    public void characters(char[] ch, int start, int length)
    {
      // zajm ns jen text uvnit elementu cena
      if (inCena)
        castka.append(ch, start, length);
    }

    // zformtovn chybovho hlen
    private String getParseExceptionInfo(SAXParseException e)
    {
      return "URI: " + e.getSystemId() + "\n" +
             "dka: " + e.getLineNumber() + " Znak: " +
             e.getColumnNumber() + "\n" +
             e.getMessage();
    }

    // obsluha chyb v dokumentu
    public void error (SAXParseException e) throws SAXException
    {
      throw new SAXException("Chyba: " + getParseExceptionInfo(e));
    }

    // obsluha fatlnch chyb v dokumentu
    public void fatalError (SAXParseException e) throws SAXException
    {
      throw new SAXException("Fatln chyba: " + getParseExceptionInfo(e));
    }

    // obsluha varovnch hlen od parseru
    public void warning (SAXParseException e)
    {
      System.out.println("Varovn: " + getParseExceptionInfo(e));
    }

  }

}